﻿#include <util/atomic.h>
#include "timer.h"


static volatile uint32_t millis_count;		// ISR Timer 0 Variable


void preSet_Timer0 ()		      // millis
{
	cli();								// Interupts ausschalten
   TCCR0A = (1 << WGM01);			// CTC Modus
	TCCR0B = 0;
	OCR0A = 77;						   // TOP, (F_CPU/PRESCALER)/1000-1
	TIMSK0 = (1 << OCIE0A);			// Compare A enable
	sei();
}


void run_Timer0 ()		
{	
	TCCR0B |= (1 << CS02) | (1 << CS00);	// Prescaler 1024
}


void stop_Timer0 ()		
{								
	TCCR0B &= ~( (1<<CS02)|(1<<CS01)|(1<<CS00) );	
}


ISR(TIMER0_COMPA_vect)
{	
	millis_count+=10;				// 10ms Intervall
}


uint32_t millis()
{
	uint32_t value = 0;
	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {
		value = millis_count;
	}
	return value;
}


void preSet_Timer1 ()		// Normal Mode 4, Servo Pulserzeugung
{
	cli();			// Interrupts ausschalten
	TCCR1A = 0;		// Reset TCCR1A Register
	TCCR1B = 0;		// Reset TCCR1B Register
	TIMSK1 = 0;		// Reset TIMSK1 Register (disable Timer Compare Interrupts)
	TCNT1 = 0;		// Start 0
	OCR1A = 19979;	// Compare & Prescaler 8 = 20µs eher 
	OCR1B = 19999;	// Compare & Prescaler 8 = 20ms 
	TIMSK1  = (1<<OCIE1A);	// enable Compare Match A ISR
	TIMSK1 |= (1<<OCIE1B);	// enable Compare Match B ISR
	sei();			// Interrupts einschalten
}  


void run_Timer1 ()			
{
	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {
		TCCR1B |= (1 << CS11);	// Prescaler 8
	}
}


void stop_Timer1 ()			// CTC, Mode 4, Servo Pulserzeugung
{
	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {
		TCCR1B &= ~( (1<<CS12)|(1<<CS11)|(1<<CS10) );	
	}
}


void preSet_Timer2()	// Normal Mode 0, OSCCAL Korrektur, Takt-Syncronisierung
{
	cli();				// Interrupts ausschalten
	TCNT2  = 0;			// Zähler nullen
	TCCR2A = 0;			// Reset Register
	TCCR2B = 0;			// Reset Register, kein WGM Bit, Normal Mode 0
	TIMSK2 = 0;			// disable Timer Interrupt
	sei();				// Interrupts einschalten
}  


void run_Timer2 ()			
{	
	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {						
		TCCR2B = (1<<CS20);					// Prescaler 1
	}
}


void stop_Timer2 ()			
{	ATOMIC_BLOCK (ATOMIC_RESTORESTATE) {									
		TCCR2B &= ~( (1<<CS22)|(1<<CS21)|(1<<CS20) );	
	}
}
